'use strict';
var co           = require('co');
var moment       = require('moment');
var express      = require('express');
var mysql        = require('@gopalroy/mysql-pool');
var TripData     = require('@gopalroy/biz-core').TripData;
var TrackService = require('./TrackService');
var ApiHelper    = require('./ApiHelper');
var Authority    = require('./Authority');
var m            = TrackApi;

function TrackApi() {
}

m.context = '/track';

m.getUserRouter = function () {
    if (!this.userRouter) {
        var router      = new express();
        this.userRouter = router;
        router.get('/', Authority.resolveVehicle, this.getTracks);
        router.get('/:id(\\d+)', Authority.resolveVehicle, this.getTrack);
        router.post('/', Authority.resolveVehicle, this.recordTrack);
    }

    return this.userRouter;
};

m.getTracks = function (req, res) {
    co(function*() {
        var args    = ApiHelper.makeQueryArgs(req.query),
            vehicle = req.incar.vehicle,
            result  = yield [TrackService.getTracksCountG({obd_code: vehicle.obd_code}),
                TrackService.getTracksG({obd_code: vehicle.obd_code}, 'id-', args.pageSize, args.nextMarker)];
        res.json({status: "success", total: result[0], tracks: result[1], count: result[1].length});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.recordTrack = function (req, res) {
    var vehicle = req.incar.vehicle;
    co(function*() {
        var trips = yield TripData.GetTripsDataG({obdCode: vehicle.obd_code, carStatus: 2}, "fireTime-", 1, null);
        if (trips.length < 1 /*|| moment(trips[0].fireTime).add(48, 'hours').diff(moment()) < 0*/) {
            return res.json({status: 'failed', errorNo: 'ILLEGAL_REQUEST', error: '没有车辆行驶数据。 请确定车辆正在行驶中, 车载设备收发数据正常。'});
        }
        var trip    = trips[0];
        var date    = moment().format('YYYY-MM-DD HH:mm:ss');
        var track   = {
            obd_code     : vehicle.obd_code,
            trip_id      : trip.tripId,
            start_time   : date,
            end_time     : date,
            start_mileage: trip.mileage,
            end_mileage  : trip.mileage,
            trip_ended   : 0
        };
        var trackId = yield TrackService.addTrackG(track);
        res.json({status: 'success', trackId: trackId});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getTrack = function (req, res) {
    co(function*() {
        var track = yield TrackService.fromIdG(req.params.id);
        if (!track) return res.json({status: 'failed', errorNo: 'NO_RECORD', error: '没有指定的轨迹信息'});
        var trip = yield mysql.pool.queryP('select * from t_obd_drive where obdCode = ? and tripId = ?', [track.obd_code, track.trip_id]);
        trip     = trip[0];
        if (req.query.refresh === 'true' && !track.trip_ended) { // 刷新数据
            if (trip.carStatus !== 2) {
                track.trip_ended = 1;
                track.end_time = trip.flameOutTime;
            } else {
                track.end_time = moment().format('YYYY-MM-DD HH:mm:ss');
            }
            yield TrackService.update({id: track.id}, {
                end_time   : track.end_time,
                end_mileage: trip.mileage,
                trip_ended : track.trip_ended
            });
        }

        var start  = moment(req.query.start || trip.start).subtract(8, 'hours').format('YYYY-MM-DD HH:mm:ss');
        var end    = moment(trip.end).subtract(8, 'hours').format('YYYY-MM-DD HH:mm:ss');
        var result = yield mysql.pool.queryP('select * from t_obd_location where obdCode = ? and tripId = ? and locationTime between ? and ?',
            [track.obd_code, track.trip_id, start, end]);
        return res.json({status: 'success', record: track, locations: result, count: result.length});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

module.exports = m;
