'use strict';
var co        = require('co');
var express   = require('express');
var moment    = require('moment');
var Authority = require('./Authority');
var ApiHelper = require('./ApiHelper');
var mysql     = require('@gopalroy/mysql-pool');

var m = TripApi;

function TripApi() {
}

m.context = '/';

m.getUserRouter = function () {
    if (!this.userRouter) {
        var router      = new express();
        this.userRouter = router;
        router.get('/trip', Authority.resolveVehicle, this.getTrips);
        router.get('/trip/:tripId(\\d+)', Authority.resolveVehicle, this.resolveTrip, this.getTrip);
        router.get('/trip/:tripId(\\d+)/path', Authority.resolveVehicle, this.resolveTrip, this.getPath);
        router.post('/trip', Authority.resolveVehicle, this.addTrip);
        router.post('/location', Authority.resolveVehicle, this.addLocation);
    }

    return this.userRouter;
};

m.getStaffRouter = function () {
    if (!this.staffRouter) {
        var router       = new express();
        this.staffRouter = router;
        router.get('/vehicle/:vehicleId(\\d+)/trip', Authority.resolveVehicle, this.getTrips);
        router.get('/trip/:tripId(\\d+)', Authority.resolveVehicle, this.resolveTrip, this.getTrip);
        router.get('/trip/:tripId(\\d+)/path', Authority.resolveVehicle, this.resolveTrip, this.getPath);
    }

    return this.staffRouter;
};

m.resolveTrip = function (req, res, next) {
    co(function*() {
        var trip = yield req.incar.vehicle.getTripByIdG(req.params.tripId);
        if (trip) req.incar.trip = trip;
        else throw new Error('错误的行程ID');
        next();
    }).catch(function (ex) {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getTrips = function (req, res) {
    co(function*() {
        var args    = ApiHelper.makeQueryArgs(req.query),
            vehicle = req.incar.vehicle,
            result  = yield [vehicle.getTripsCountG(), vehicle.getTripsG(args.pageSize, args.nextMarker)];
        res.json({status: "success", total: result[0], trips: result[1], count: result[1].length});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getTrip = function (req, res) {
    res.json({status: 'success', trip: req.incar.trip});
};

m.getPath = function (req, res) {
    var args = ApiHelper.makeQueryArgs(req.query);
    var trip = req.incar.trip;
    co(function*() {
        var result = yield [trip.getPathPointCountG(args.filter), trip.getPathG(args.filter, args.pageSize, args.nextMarker)];
        res.json({status: 'success', total: result[0], paths: result[1]});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.addTrip = function (req, res) {
    var vehicle = req.incar.vehicle;
    var fireTime= req.body.fireTime;
    var flameOutTime = req.body.flameOutTime;
    if (!fireTime || !moment(fireTime).isValid() || !flameOutTime || !moment(flameOutTime).isValid()) {
        return res.json({
            status : 'failed',
            errorNo: 'ILLEGAL_PARAM_FORMAT',
            error  : '点火时间或熄火时间格式不对, 正确格式: 2016-12-06 12:05:01'
        });
    }
    co(function*() {
        var tripId = 1;
        var result = yield mysql.pool.queryP('select max(tripId) + 1 as tripId from t_obd_drive where obdCode = ?', [vehicle.obd_code]);
        if (result.length > 0 && result[0].tripId) tripId = result[0].tripId;
        yield mysql.pool.queryP('insert into t_obd_drive (obdCode, tripId, fireTime, flameOutTime, carStatus) value (?, ?, ?, ?, 2)',
            [vehicle.obd_code, tripId, fireTime, flameOutTime]);
        return res.json({status: 'success', tripId: tripId});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.addLocation = function (req, res) {
    var vehicle      = req.incar.vehicle;
    var tripId       = req.body.tripId;
    var longitude    = req.body.longitude;
    var latitude     = req.body.latitude;
    var locationTime = moment(req.body.locationTime);

    if (!locationTime.isValid())
        return res.json({
            status : 'failed',
            errorNo: 'ILLEGAL_PARAM_FORMAT',
            error  : '时间格式不对, 正确格式: 2016-12-06 12:05:01'
        });
    if (!longitude || !longitude.startsWith('E') || !latitude || !latitude.startsWith('N'))
        return res.json({status: 'failed', errorNo: 'ILLEGAL_PARAM_FORMAT', error: '经纬度应标注东经E北纬N'});

    co(function*() {
        var trip = yield mysql.pool.queryP('select * from t_obd_drive where obdCode = ? and tripId = ?', [vehicle.obd_code, tripId]);
        if (trip.length < 1) return res.json({status: 'failed', errorNo: 'NO_TRIP_RECORD', error: '没有指定的行程记录'});
        trip = trip[0];

        if (locationTime.diff(trip.fireTime) < 0 || locationTime.diff(trip.flameOutTime) > 0) {
            return res.json({status: 'failed', errorNo: 'ILLEGAL_PARAM_VALUE', error: '定位时间必须位于行程起止时间之内'});
        }

        var result = yield mysql.pool.queryP('insert into t_obd_location (obdCode, tripId, locationTime, longitude, latitude) value (?,?,?,?,?)',
            [vehicle.obd_code, tripId, locationTime.subtract(8, 'hours').format('YYYY-MM-DD HH:mm:ss'), longitude, latitude]);
        var inserted = yield mysql.pool.queryP('select * from t_obd_location where id = ?', [result.insertId]);
        return res.json({status: 'success', location: inserted[0]});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

module.exports = m;
