'use strict';
var co                = require('co');
var fs                = require('fs');
var path              = require('path');
var express           = require('express');
var moment            = require('moment');
var formidable        = require('formidable');
var ApiHelper         = require('./ApiHelper');
var AppVersionService = require('./AppVersionService');

var m = VersionApi;
function VersionApi() {
}

m.context = '/app';

m.getUserRouter = function () {
    if (!this.userRouter) {
        var r           = new express();
        this.userRouter = r;
        r.get('/latest', this.getLatestVersion);
        r.get('/download', this.download);
    }

    return this.userRouter;
};

m.getStaffRouter = function () {
    if (!this.staffRouter) {
        var r            = new express();
        this.staffRouter = r;
    }
    r.get('/', this.getApps);
    r.get('/download', this.download);
    r.get('/:short_name', this.getApp);
    r.get('/:short_name/latest', this.getLatestVersion);
    r.get('/:short_name/versions', this.getVersions);
    r.post('/', this.addApp);
    r.post('/upload', this.uploadFile);
    r.post('/version', this.addVersion);
    return this.staffRouter;
};

m.addApp = function (req, res) {
    var params = ['name', 'short_name', 'app_type', 'description'];
    var app    = {};
    params.forEach(p => {
        if (!req.body[p]) return res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAM', error: '缺少参数: ' + p});
        app[p] = req.body[p];
    });
    app.created_at = moment().format('YYYY-MM-DD HH:mm:ss');

    co(function*() {
        app.id = yield AppVersionService.addAppG(app);
        return res.json({status: 'success', app: app});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getApps = function (req, res) {
    var args = ApiHelper.makeQueryArgs(req.query);
    co(function*() {
        var result = yield [AppVersionService.getAppsCountG(args.filter),
            AppVersionService.getAppsG(args.filter, 'id-', args.pageSize, args.nextMarker)];
        return res.json({status: 'success', count: result[1].length, total: result[0], items: result[1]});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getApp = function (req, res) {
    var short_name = req.params.short_name;

    co(function*() {
        var result = yield AppVersionService.getAppsG({short_name: short_name}, null, 1, 1);
        return res.json({status: 'success', app: result[0] || null});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.addVersion = function (req, res) {
    var params = ['short_name', 'version', 'file_path', 'file_size', 'description'];
    var version = {};

    params.forEach(p => {
        if (!req.body[p]) return res.json({status: 'failed', errorNo: 'MISSING_PARAP', error: '缺少参数: ' + p});
        version[p] = req.body[p];
    });
    version.published_time = moment().format('YYYY-MM-DD HH:mm:ss');

    co(function*() {
        var app = yield AppVersionService.getAppsG({short_name: version.short_name});
        if (!app.length) return res.json({status: 'failed', errorNo: 'WRONG_SHORT_NAME', error: '指定short_name的App不存在'});
        version.id = yield AppVersionService.addVersionG(version);
        res.json({status: 'success', data: version});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.uploadFile = function (req, res) {
    var form = new formidable.IncomingForm();
    var root = path.dirname(require.main.filename);

    form.parse(req, function (err, fields, files) {
        var file = files.app;
        if (!file) return res.json({status: 'failed', errorNo: 'NO_UPLOAD_FILE', error: '没有上传文件'});
        if (file.size > 100 * 1024 * 1024) {
            fs.unlink(file.path);
            return res.json({status: 'failed', errorNo: 'UPLOAD_SIZE_EXCEED', error: '上传文件大小超出限制, 最大允许上传100MB。'});
        }

        var filename = moment().format('YYYYMMDD') + Math.random().toString().slice(2, 6) + file.name.match(/\.[^.]*$/)[0];
        var dist     = path.join(root, '/upload/app');
        if (!fs.existsSync(dist)) m.mkdirsSync(dist);
        fs.rename(file.path, path.join(root, '/upload/app', filename), function (err) {
            if (err) {
                console.log(__filename, err.stack);
                return res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: err.message});
            }
            res.json({status: 'success', path: filename, size: file.size});
        });
    });
};

m.getLatestVersion = function (req, res) {
    var short_name = req.params.short_name || req.query.short_name;
    if (!short_name) return res.json({status: 'failed', errorNo: 'MISSING_PARAM', error: '请指定app名称'});

    co(function*() {
        var result = yield AppVersionService.getVersionsG({short_name: short_name}, 1);
        return res.json({status: 'success', version: result[0] || null});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getVersions = function (req, res) {
    var short_name = req.params.short_name;

    co(function*() {
        var result = yield [AppVersionService.getVersionsCountG({short_name: short_name}),
            AppVersionService.getVersionsG({short_name: short_name}, req.query.pageSize, req.query.nextMarker)];
        return res.json({status: 'success', total: result[0], count: result[1].length, items: result[1]});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.deleteVersion = function (req, res) {
    co(function*() {
        yield AppVersionService.deleteVersionG(req.params.id);
        return res.json({status: 'success'});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.download = function (req, res) {
    var filePath = path.join(path.dirname(require.main.filename), '/upload/app', req.query.filename);
    fs.exists(filePath, function (exist) {
        if (exist) {
            res.setHeader('Accept-Range', 'bytes');
            res.writeHead(200, 'Partial Content', {
                'Content-Disposition': 'attachment; filename=' + req.query.filename,
                'Content-Type'       : 'application/octet-stream'
            });
            fs.stat(filePath, function (error, state) {
                if (error) res.json({status: 'failed', error: error.message});
                var fileSize    = state.size;
                var fReadStream = fs.createReadStream(filePath, {
                    encoding  : 'binary',
                    bufferSize: 1024 * 1024,
                    start     : 0,
                    end       : fileSize
                });
                fReadStream.on('data', function (chunk) {
                    res.write(chunk, 'binary');
                });
                fReadStream.on('end', function () {
                    res.end();
                });
            });
        } else {
            res.status(404).json({status: 'failed', errorNo: 'RESOURCE_NOT_EXIST', error: '文件不存在'});
        }
    });
};

m.mkdirsSync = function (p) {
    try {
        fs.mkdirSync(p);
    } catch (err) {
        if (err.code === 'ENOENT') {
            this.mkdirsSync(path.dirname(p));
            this.mkdirsSync(p);
        } else throw err;
    }
};

module.exports = m;