import {Injector} from 'ngES6';
import moment from 'moment';
import './vehicle.scss';

export default class VehicleCtrl extends Injector {
    static $inject   = ['$scope', '$rootScope', '$state', '$stateParams', '$timeout', 'VehicleService', 'UserService', 'CareService'];
    static $template = require('./vehicle.html');

    constructor(...args) {
        super(...args);
        const {$scope, $rootScope, $stateParams, VehicleService} = this.$injected;
        this.attachMethodsTo($scope);
        (async function () {
            let vehicle, vehicleP = await VehicleService.getDetail($stateParams.id);
            if (vehicleP.data) {
                if (vehicleP.data.status === 'success') vehicle = vehicleP.data.vehicle;
                else throw new Error(vehicleP.data.error);
            } else throw new Error('访问网络失败');

            const series       = await $rootScope.getSeries(vehicle.brand);
            vehicle.brandName  = $rootScope.brands.find(i => i.brandCode === vehicle.brand).brand;
            vehicle.seriesName = series.find(i => i.seriesCode === vehicle.series).series;
            return vehicle;
        })().then(vehicle => {
            $scope.vehicle = vehicle;
            this.getAnalysis();
        }).catch(ex => $scope.notice = ex.message);
    }

    getAnalysis() {
        const {$scope, VehicleService} = this.$injected;
        const vehicle    = $scope.vehicle;
        let filter       = $scope.analysisFilter || ($scope.analysisFilter = {
            start: moment().subtract(30, 'days').format('YYYY-MM-DD'),
            end  : moment().format('YYYY-MM-DD'),
        });
        $scope.showTable = 'analysis';
        VehicleService.getSpeedDistribution(vehicle.id, filter.start, filter.end).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.speedDistribution = {label: [], data: []};
                result.data.forEach(i => {
                    let label;
                    switch (i.speed) {
                    case 1:
                        label = '小于45Km/h';
                        break;
                    case 45:
                        label = '45-90Km/h';
                        break;
                    case 90:
                        label = '90-255Km/h';
                        break;
                    case 255:
                        label = '大于255Km/h';
                        break;
                    default:
                        label = '';
                    }
                    $scope.speedDistribution.label.push(label);
                    $scope.speedDistribution.data.push(i.distance);
                });
            }
        });
        VehicleService.getOilUsage(vehicle.id, filter.start, filter.end).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.oilUsage = {
                    label: [],
                    data : [[]],
                    total: {oil: result.total_oil, mileage: result.total_mileage, avgOil: result.avg_oil},
                };
                result.data.forEach(i => {
                    $scope.oilUsage.label.push(moment(i.date).format('YYYY-MM-DD'));
                    $scope.oilUsage.data[0].push(i.avg_oil);
                });
            }
        });
        VehicleService.getDrivingHabit(vehicle.id, filter.start, filter.end).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.habits = {label: [], data: [[], [], []], total: result.total};
                result.data.forEach(i => {
                    $scope.habits.label.push(moment(i.date).format('YYYY-MM-DD'));
                    $scope.habits.data[0].push(i.speedUp);
                    $scope.habits.data[1].push(i.speedDown);
                    $scope.habits.data[2].push(i.sharpTurn);
                });
            }
        });
    }

    getTrips() {
        const {$scope, VehicleService} = this.$injected;
        const vehicle    = $scope.vehicle;
        let pager        = $scope.tripPager || ($scope.tripPager = {page: 1, totalItems: 0, pageSize: 15});
        $scope.showTable = 'trips';
        VehicleService.getTrips(vehicle.id, pager.page, pager.pageSize).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.trips     = result.trips;
                pager.totalItems = result.total;
            }
        });
    }

    getTracks() {
        const {$scope, VehicleService} = this.$injected;
        const vehicle    = $scope.vehicle;
        let pager        = $scope.trackPager || ($scope.trackPager = {page: 1, totalItems: 0, pageSize: 15});
        $scope.showTable = 'tracks';
        VehicleService.getTracks(vehicle.id, pager.page, pager.pageSize).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.tracks    = result.tracks;
                pager.totalItems = result.total;
            }
        });
    }

    getReports(type) {
        const {$scope, VehicleService} = this.$injected;
        const vehicle     = $scope.vehicle;
        $scope.showTable  = 'reports';
        $scope.reportType = type || $scope.reportType || 'week';
        let pager         = $scope.reportPager || ($scope.reportPager = {page: 1, totalItems: 0, pageSize: 15});
        VehicleService.getReports(vehicle.id, $scope.reportType, pager.page, pager.pageSize).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.reports      = result.report;
                pager.totalItems    = result.total;
                $scope.reportMax    = result.max;
                $scope.reportAllMax = result.allMax;
                $scope.reportAllAvg = result.allAvg;
            }
        });
    }

    getDetects() {
        const {$scope, VehicleService} = this.$injected;
        const vehicle    = $scope.vehicle;
        $scope.showTable = 'detects';
        let pager        = $scope.detectPager || ($scope.detectPager = {page: 1, totalItems: 0, pageSize: 15});
        VehicleService.getDetects(vehicle.id, pager.page, pager.pageSize).then(res => {
            const result = this.checkResult(res);
            if (result) {
                result.items.forEach(i => i.description = JSON.parse(i.fault).map(v => v.code + ':' + v.desc).join('; '));
                $scope.detects   = result.items;
                pager.totalItems = result.total;
            }
        });
    }

    getCares() {
        const {$scope, $timeout, CareService} = this.$injected;
        const vehicle    = $scope.vehicle;
        $scope.showTable = 'cares';
        let pager        = $scope.carePager || ($scope.carePager = {page: 1, totalItems: 0, pageSize: 15});
        let _this = this;
        (async function() {
            const config = await CareService.getConfig();
            const res = await CareService.getCares(vehicle.id, pager.page, pager.pageSize);
            const result = _this.checkResult(res);
            if (!result) throw new Error();
            result.cares.forEach(i => i.content = i.items.map(v => config.find(c => c.id === v.cfg_id).name).join(', '));
            return result;
        })().then(result => {
            $timeout(() => {
                $scope.cares     = result.cares;
                pager.totalItems = result.total;
            });
        });
    }

    checkResult(res) {
        const {$scope} = this.$injected;
        if (res.data) {
            if (res.data.status === 'success') return res.data;
            $scope.notice = res.data.error;
        } else $scope.notice = '访问网络失败';

        return null;
    }

}
